@@ -53,8 +53,8 @@ def _print_bucket_summary(bucket_stats: List[Dict[str, Any]]) -> None:
     print()
 
 
-def _plot_bucket_thumbnails(bucket_stats: List[Dict[str, Any]], viz_data: Dict) -> None:
-    """Show thumbnail grids for each bucket - 10x10 grid with up to 100 images per bucket."""
+def _plot_bucket_thumbnails(bucket_stats: List[Dict[str, Any]], viz_data: Dict, show_progress: bool = True) -> None:
+    """Show thumbnail grids for each bucket - 5x5 grid with up to 25 images per bucket."""
     try:
         import matplotlib.pyplot as plt
         import numpy as np
@@ -78,8 +78,8 @@ def _plot_bucket_thumbnails(bucket_stats: List[Dict[str, Any]], viz_data: Dict)
     cols = int(np.ceil(np.sqrt(n_buckets)))
     rows = int(np.ceil(n_buckets / cols))
     
-    # Create figure
-    fig, axes = plt.subplots(rows, cols, figsize=(cols * 5, rows * 5))
+    # Create figure with extra height for titles
+    fig, axes = plt.subplots(rows, cols, figsize=(cols * 5, rows * 6))
     
     # Ensure axes is always a 2D array for consistent access
     if rows == 1 and cols == 1:
@@ -89,8 +89,13 @@ def _plot_bucket_thumbnails(bucket_stats: List[Dict[str, Any]], viz_data: Dict)
     elif cols == 1:
         axes = axes.reshape(-1, 1)
     
-    # Process each bucket
-    for bucket_idx, bucket_data in enumerate(sorted_buckets):
+    # Process each bucket with progress bar
+    if show_progress:
+        bucket_iter = tqdm(enumerate(sorted_buckets), desc=" - Creating thumbnail grids", total=len(sorted_buckets))
+    else:
+        bucket_iter = enumerate(sorted_buckets)
+    
+    for bucket_idx, bucket_data in bucket_iter:
         # Get images for this bucket (reverse index since we sorted)
         original_bucket_idx = len(sorted_buckets) - 1 - bucket_idx
         bucket_images = []
@@ -98,21 +103,21 @@ def _plot_bucket_thumbnails(bucket_stats: List[Dict[str, Any]], viz_data: Dict)
             if assigned_bucket == original_bucket_idx:
                 bucket_images.append(all_paths[path_idx])
         
-        # Create 10x10 thumbnail grid
-        def create_bucket_grid(images, max_images=100):
-            """Create a 10x10 grid of thumbnails from bucket images."""
+        # Create 5x5 thumbnail grid
+        def create_bucket_grid(images, max_images=25):
+            """Create a 5x5 grid of thumbnails from bucket images."""
             if not images:
                 return np.ones((300, 300, 3), dtype=np.uint8) * 220  # Gray placeholder
             
-            # Take first 100 images (not random) for consistent display
+            # Take first 25 images (not random) for consistent display
             sample_images = images[:max_images]
             
-            # Create 10x10 grid (300x300 pixels, each thumbnail 30x30)
+            # Create 5x5 grid (300x300 pixels, each thumbnail 60x60)
             grid_img = np.ones((300, 300, 3), dtype=np.uint8) * 255  # White background
-            thumb_size = 30
+            thumb_size = 60
             
-            for idx, img_path in enumerate(sample_images[:100]):  # Ensure max 100
-                if idx >= 100:
+            for idx, img_path in enumerate(sample_images[:25]):  # Ensure max 25
+                if idx >= 25:
                     break
                     
                 try:
@@ -123,9 +128,9 @@ def _plot_bucket_thumbnails(bucket_stats: List[Dict[str, Any]], viz_data: Dict)
                         img_thumb = img.resize((thumb_size, thumb_size), Image.Resampling.LANCZOS)
                         img_array = np.array(img_thumb, dtype=np.uint8)
                         
-                        # Calculate position in 10x10 grid
-                        row = idx // 10
-                        col = idx % 10
+                        # Calculate position in 5x5 grid
+                        row = idx // 5
+                        col = idx % 5
                         y_start = row * thumb_size
                         x_start = col * thumb_size
                         y_end = y_start + thumb_size
@@ -148,7 +153,15 @@ def _plot_bucket_thumbnails(bucket_stats: List[Dict[str, Any]], viz_data: Dict)
         # Create and display thumbnail grid
         grid_img = create_bucket_grid(bucket_images)
         ax.imshow(grid_img)
-        ax.axis('off')
+        ax.set_title(str(bucket_idx + 1), fontsize=12, pad=2)
+        
+        # Add thin grey border around the grid
+        for spine in ax.spines.values():
+            spine.set_visible(True)
+            spine.set_color('lightgrey')
+            spine.set_linewidth(1)
+        ax.set_xticks([])
+        ax.set_yticks([])
     
     # Hide unused subplots
     for i in range(n_buckets, rows * cols):
@@ -156,10 +169,10 @@ def _plot_bucket_thumbnails(bucket_stats: List[Dict[str, Any]], viz_data: Dict)
         col = i % cols
         axes[row, col].axis('off')
     
-    plt.suptitle('Bucket Thumbnails: Visual Similarity Groups (10x10 grid, max 100 images per bucket)', 
-                 fontsize=14, y=0.95)
-    plt.tight_layout()
-    plt.subplots_adjust(top=0.9)
+    plt.suptitle('Bucket thumbnails: visual similarity groups (5x5 grid, max 25 images per bucket)', 
+                 fontsize=14, y=0.96)
+    plt.tight_layout(pad=3.0)
+    plt.subplots_adjust(top=0.85, hspace=0.4)
     plt.show()
 
 
@@ -179,7 +192,7 @@ def _plot_bucket_distribution(bucket_stats: List[Dict[str, Any]]) -> None:
     # Sort buckets by original size (largest first)
     sorted_buckets = sorted(bucket_stats, key=lambda x: x['original_size'], reverse=True)
     
-    bucket_names = [f"Bucket {i+1}" for i in range(len(sorted_buckets))]
+    bucket_names = [str(i+1) for i in range(len(sorted_buckets))]
     kept_counts = [b['kept'] for b in sorted_buckets]
     excluded_counts = [b['excluded'] for b in sorted_buckets]
     
@@ -193,22 +206,13 @@ def _plot_bucket_distribution(bucket_stats: List[Dict[str, Any]]) -> None:
     bars_excluded = ax.bar(x, excluded_counts, width, bottom=kept_counts, 
                           label='Excluded', color='#CD5C5C', alpha=0.8)
     
-    ax.set_xlabel('Visual Similarity Buckets (sorted by size)')
-    ax.set_ylabel('Number of Images')
-    ax.set_title('Bucket Distribution: Kept vs Excluded')
+    ax.set_xlabel('Visual similarity buckets (sorted by size)')
+    ax.set_ylabel('Number of images')
+    ax.set_title('Bucket distribution: kept vs excluded')
     ax.set_xticks(x)
     ax.set_xticklabels(bucket_names, rotation=45, ha='right')
     ax.legend()
     
-    # Add percentage labels on bars
-    for i, (kept, excluded) in enumerate(zip(kept_counts, excluded_counts)):
-        total = kept + excluded
-        if total > 0:
-            kept_pct = (kept / total) * 100
-            # Only show percentage if bar is tall enough
-            if kept > total * 0.1:
-                ax.text(i, kept/2, f'{kept_pct:.0f}%', ha='center', va='center', 
-                       fontweight='bold', color='white')
     
     plt.tight_layout()
     plt.show()
@@ -245,7 +249,7 @@ def sample_diverse(
         random_seed: Random seed for reproducibility
         show_summary: Whether to print bucket distribution summary (default: True)
         show_distribution: Whether to show bucket distribution chart (default: False)
-        show_thumbnails: Whether to show 10x10 thumbnail grids for each bucket (default: False)
+        show_thumbnails: Whether to show 5x5 thumbnail grids for each bucket (default: False)
         
     Returns:
         List of exactly target_count selected image paths
@@ -262,7 +266,10 @@ def sample_diverse(
     
     n_images = len(image_paths)
     
+    # Early exit: if we want everything or more, just return all paths
     if target_count >= n_images:
+        if show_progress:
+            print(f"Target count ({target_count}) >= available images ({n_images}), returning all images")
         return [str(p) for p in image_paths]
     
     if target_count <= 0:
@@ -272,11 +279,14 @@ def sample_diverse(
         n_workers = min(4, max(1, 4))  # Default to 4 workers
     
     if show_progress:
-        print(f"Selecting {target_count} from {n_images} images using fast grid selection")
+        print(f"Selecting {target_count} from {n_images} images...")
+    
+    # Sort all input paths naturally to preserve camera/folder structure globally
+    if show_progress:
+        print(" - Sorting paths...")
+    sorted_image_paths = natsorted([str(p) for p in image_paths])
     
     # Step 1: Compute hashes in parallel
-    if show_progress:
-        print("Computing image hashes...")
     
     def compute_hash(path):
         try:
@@ -299,6 +309,9 @@ def sample_diverse(
                 r, g, b = img_array[:,:,0], img_array[:,:,1], img_array[:,:,2]
                 color_variance = np.var([np.mean(r), np.mean(g), np.mean(b)])
                 
+                # Average color (dominant color theme)
+                avg_r, avg_g, avg_b = np.mean(r), np.mean(g), np.mean(b)
+                
                 # Brightness level
                 brightness = np.mean(img_array)
                 
@@ -307,6 +320,7 @@ def sample_diverse(
                     'dhash': dhash,
                     'ahash': ahash,
                     'color_variance': color_variance,
+                    'avg_color': (avg_r, avg_g, avg_b),
                     'brightness': brightness
                 }
                 
@@ -318,10 +332,10 @@ def sample_diverse(
     hashes = []
     
     with ThreadPoolExecutor(max_workers=n_workers) as executor:
-        futures = [executor.submit(compute_hash, path) for path in image_paths]
+        futures = [executor.submit(compute_hash, path) for path in sorted_image_paths]
         
         if show_progress:
-            futures_iter = tqdm(futures, desc="Hashing images")
+            futures_iter = tqdm(futures, desc=" - Hashing images")
         else:
             futures_iter = futures
         
@@ -338,7 +352,7 @@ def sample_diverse(
     
     # Step 2: Create diversity grid using multi-dimensional features
     if show_progress:
-        print("Creating diversity grid with enhanced visual features...")
+        print(" - Creating diversity grid...")
     
     bucket_keys = []
     
@@ -346,19 +360,28 @@ def sample_diverse(
         dhash = features['dhash']
         ahash = features['ahash']
         color_var = features['color_variance']
+        avg_r, avg_g, avg_b = features['avg_color']
         brightness = features['brightness']
         
         # Convert dhash to bits (structural features)
         dhash_bits = np.array(dhash.hash).flatten()
         ahash_bits = np.array(ahash.hash).flatten()
         
-        # Use fewer bits from dhash for structure (reduce from 4 to 3)
+        # Use 2 bits from center of dhash (where animals typically are)
+        # For 8x8 hash (64 bits), center would be around indices 27, 28, 35, 36
+        # We'll take 2 strategically placed center bits
         total_dhash_bits = len(dhash_bits)
-        if total_dhash_bits >= 3:
-            dhash_indices = [0, total_dhash_bits//3, 2*total_dhash_bits//3]
-            structure_bits = tuple(dhash_bits[i] > 0 for i in dhash_indices)
+        if total_dhash_bits >= 64:  # 8x8 hash
+            # Center-left and center-right of the hash (row 3-4, col 3-4)
+            center_indices = [27, 36]  # Strategic center positions
+            structure_bits = tuple(dhash_bits[i] > 0 for i in center_indices)
+        elif total_dhash_bits >= 16:  # 4x4 hash
+            # Center positions for smaller hash
+            center_indices = [5, 10]  # Center positions in 4x4
+            structure_bits = tuple(dhash_bits[i] > 0 for i in center_indices if i < total_dhash_bits)
         else:
-            structure_bits = tuple(dhash_bits > 0)
+            # Very small hash, just use first 2 bits
+            structure_bits = tuple(dhash_bits[:2] > 0) if total_dhash_bits >= 2 else tuple(dhash_bits > 0)
         
         # Use only 1 bit from ahash for basic brightness pattern
         brightness_bit = (ahash_bits[0] > 0,) if len(ahash_bits) >= 1 else ()
@@ -369,10 +392,24 @@ def sample_diverse(
         # Simplified brightness (0=dark, 1=bright)
         bright_bucket = 0 if brightness < 128 else 1  # Binary: dark vs bright
         
+        # Dominant color theme detection (for colored images only)
+        if color_bucket == 1:  # Only apply to colored images
+            # Determine dominant color channel
+            if avg_r > avg_g and avg_r > avg_b and avg_r > 140:
+                color_theme = 0  # Red-dominant (warm/orange/brown scenes)
+            elif avg_g > avg_r and avg_g > avg_b and avg_g > 140:
+                color_theme = 1  # Green-dominant (forest/grass scenes)  
+            elif avg_b > avg_r and avg_b > avg_g and avg_b > 140:
+                color_theme = 2  # Blue-dominant (sky/snow/water scenes)
+            else:
+                color_theme = 3  # Neutral/mixed colors
+        else:
+            color_theme = 0  # Grayscale images get default theme
+        
         # Combine all features into bucket key
-        # 3 structure + 1 brightness + 1 color + 1 overall_brightness = 6 dimensions
-        # Max buckets: 2^3 × 2^1 × 2 × 2 = 8 × 2 × 2 × 2 = 64 buckets
-        bucket_key = structure_bits + brightness_bit + (color_bucket, bright_bucket)
+        # 2 center structure + 1 brightness + 1 color + 1 overall_brightness + 2 color_theme = 7 dimensions  
+        # Max buckets: 2^2 × 2^1 × 2 × 2 × 4 = 4 × 2 × 2 × 2 × 4 = 128 buckets (but many combinations won't exist)
+        bucket_key = structure_bits + brightness_bit + (color_bucket, bright_bucket, color_theme)
         bucket_keys.append(bucket_key)
     
     # Group images by bucket
@@ -382,25 +419,14 @@ def sample_diverse(
             buckets[key] = []
         buckets[key].append(i)
     
-    # Sort indices within each bucket by natural path order
-    # This ensures folder structure is preserved during stride sampling
-    for key, indices in buckets.items():
-        # Create list of (path, index) pairs, sort by path using natsort, extract indices
-        path_index_pairs = [(valid_paths[i], i) for i in indices]
-        sorted_pairs = natsorted(path_index_pairs, key=lambda x: x[0])
-        buckets[key] = [pair[1] for pair in sorted_pairs]
+    # Note: Paths are already naturally sorted globally, so bucket order preserves camera/folder structure
     
     if show_progress:
-        print(f"Grouped into {len(buckets)} diversity buckets with enhanced visual features")
-        print(f"Features: structure (3 bits) + brightness pattern (1 bit) + color type (2 levels) + brightness level (2 levels)")
-        
         # Show bucket size distribution
         bucket_sizes = [len(indices) for indices in buckets.values()]
-        print(f"Bucket sizes: min={min(bucket_sizes)}, max={max(bucket_sizes)}, avg={sum(bucket_sizes)/len(bucket_sizes):.1f}")
+        print(f" - Grouped into {len(buckets)} diversity buckets... (min={min(bucket_sizes)}, max={max(bucket_sizes)}, avg={sum(bucket_sizes)/len(bucket_sizes):.1f})")
     
     # Step 3: Diversity-preserving selection
-    if show_progress:
-        print(f"Using diversity-first with proportional distribution across {len(buckets)} buckets")
     
     # Sort buckets by size (largest first)
     bucket_list = [(key, indices) for key, indices in buckets.items()]
@@ -494,7 +520,7 @@ def sample_diverse(
     selected_paths = [valid_paths[i] for i in selected_indices]
     
     if show_progress:
-        print(f"✓ Selected {len(selected_paths)} images with diversity preservation")
+        print(f" - Selected {len(selected_paths)} images with diversity preservation!")
     
     # Show visualizations if requested
     if show_summary or show_distribution or show_thumbnails:
@@ -532,7 +558,7 @@ def sample_diverse(
                 'all_paths': valid_paths
             }
             
-            _plot_bucket_thumbnails(bucket_stats, viz_data)
+            _plot_bucket_thumbnails(bucket_stats, viz_data, show_progress)
     
     return selected_paths
 
